MQSeries stress testing
	c Xephon 1999



JPanel bottom = new JPanel();
    dlgContents.add(bottom, BorderLayout.SOUTH);
    bottom.setLayout(new FlowLayout());
    JButton btnOk = new JButton("Ok");
    bottom.add(btnOk);
    btnOk.addActionListener( new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        iMessagedlg.setVisible(false);
      }
    });
    iMessagedlg.pack();
    iMessagedlg.setVisible(true);
  }

/**
*   Return the singleton's instance
*/
  public static synchronized MQConstants getInstance() {

    if (cInstance == null) {

      try {
        cInstance = new MQConstants(); // Swing styles must be
                                       // re-instantiated.
        ObjectInputStream ois = new ObjectInputStream( new 
                                  FileInputStream(PERSISTENCY_NAME) );
        cInstance = (MQConstants) ois.readObject();
        cInstance.addListeners();
        ois.close();
      } catch (Exception e) {
        cInstance = new MQConstants();
      }

      cInstance.setVisible(true);
      cInstance.repaint();
    }
    return cInstance;
  }

/**
*   Store a reference to any classes interested in the completion 
*   of the gathering of constants
*/
  public void addConstantsListener(ConstantsListener lis) {
    iListeners.addElement(lis);
  }
/**
*   Notify listeners that all constants have now been gathered
*/
  public void notifyConstantsListeners() {
    for (Enumeration enum = iListeners.elements(); 
                                    enum.hasMoreElements(); ) {
      ((ConstantsListener) enum.nextElement()).constantsAvailable();
    }
  }
}


package com.dmitri.mqstress;
/**
*   @author Dmitri
*  Simple interface to notify that constants are now ready for use
*/
public interface ConstantsListener {

  public void constantsAvailable();
}


package com.dmitri.mqstress;
import java.awt.event.*;
import com.sun.java.swing.*;
import java.awt.*;
/**
*   @author Dmitri
*
*   This is the top-level class of the MQStressTester package. It 
*   has overall responsibility for other classes.
*
*  Its primary actions are:
*
*  1. Instantiate the constants window and wait for it to finish 
*     its information gathering and processing.
*  2. Instantiate an appropriate number of MQStressThreads.
*  3. Present the user with controls for starting and stopping 
*     the test.
*  4. Control the starting and stopping of the stress test.
*/
public class MQStressTester implements ConstantsListener {

  private MQConstants iConstants = MQConstants.getInstance();

  private JFrame iFrmResults = new JFrame("MQStress Results");
  private JButton iBtnStart = new JButton("Start");
  private JButton iBtnStop = new JButton("Stop");
  private JButton iBtnExit = new JButton("Exit");

  private static long cRepaintInterval = 1000l;
  private MQStressThread [] iThreads;    // An array of test threads

/**
*   A simple constructor to register an interest in the completion 
*   of MQConstants. As we are dependent on constants being gathered, 
*   it is necessary to wait until they are all in place.
*/
  public MQStressTester() {
    super();
    iConstants.addConstantsListener(this);
  }

/**
*   This method starts things off when the MQConstants class has 
*   finished gathering data.
*/
  public void constantsAvailable() {
    begin();
  }

/**
*   This method initializes the window for displaying the results 
*   of the stress test while in progress.
*/
  private void begin() {

    int numberOfThreads = iConstants.getNumberOfInstances();
    iThreads = new MQStressThread[numberOfThreads];

    JPanel pnlContents = new JPanel();
    JPanel pnlMain = new JPanel();
    iFrmResults.getContentPane().add(pnlContents);
    pnlContents.setBorder(BorderFactory.createEmptyBorder(
                                                    10, 10, 10, 10));
    pnlContents.setLayout(new BorderLayout());
    pnlContents.add(pnlMain);
    pnlMain.setLayout(new FlowLayout());

    for (int i = 0; i < numberOfThreads; i++) {

      iThreads[i] = new MQStressThread();
      pnlMain.add(iThreads[i].getRenderer());
    }

    JPanel pnlButtons = new JPanel();
    pnlButtons.setLayout(new FlowLayout());
    pnlContents.add(pnlButtons, BorderLayout.SOUTH);
    pnlButtons.add(iBtnStart);
    pnlButtons.add(iBtnStop);
    pnlButtons.add(iBtnExit);

    addListeners();
    iFrmResults.pack();
    iFrmResults.setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);
    iFrmResults.setVisible(true);
  }

/**
*   This method adds inner class listeners to all buttons.
*/
  private void addListeners() {

    iBtnStart.addActionListener( new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        for (int i = 0; i < iThreads.length; i++) {
          new Thread( iThreads[i] ).start();
        }
      }
    });

    iBtnStop.addActionListener( new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        for (int i = 0; i < iThreads.length; i++) {
          iThreads[i].stopTest();
        }
      }
    });

    iBtnExit.addActionListener( new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        for (int i = 0; i < iThreads.length; i++) {
          iThreads[i].exit();
          iFrmResults.setVisible(false);
          System.exit(0);
        }
      }
    });
  }

/**
*   This is a standard main method to instantiate this class.
*/
  public static void main(String[] args) {
    new MQStressTester();
  }
}


package com.dmitri.mqstress;
import com.ibm.mq.*;
/**
*   @author Dmitri
*
*   This is a simple class that interfaces to MQSeries, putting 
*   and getting simple messages. It doesn't act on messages. This 
*   class is not capable of recovering from MQSeries errors (such 
*   as connection lost).
*/
public class MQService {

  private MQConstants iConstants = MQConstants.getInstance();
  private MQQueue iRequestQueue = null;
  private MQQueue iReplyQueue = null;
  private MQGetMessageOptions iGmo = new MQGetMessageOptions();
  private MQPutMessageOptions iPmo = new MQPutMessageOptions();
  private MQQueueManager iQmgr;

/**
*  Initializes a new service to MQSeries
*/
  public MQService() {
    System.runFinalizersOnExit(true);
  }

/**
 * This method sets up the default message parameters. Some of these 
 * are hard-coded, others are obtained from the constants class.
 */
  private MQMessage applyDefaults(String msg) {

    try {
      MQMessage mqmess = new MQMessage();
      mqmess.replyToQueueName = iReplyQueue.name;
      mqmess.format = MQC.MQFMT_STRING;
      mqmess.writeString(msg);

      return mqmess;

    } catch (java.io.IOException e) {
      e.printStackTrace();
    }
    return null;
  }

/**
 * This method initializes the MQ component of the service.
 * This includes connection to the queue manager as well as the 
 * opening queues.
 *
 * Note: for future expansion, it may be a good idea to measure 
 * connection times etc, as this takes an appreciable time in a 
 * stressed system.
 */
  public void mqStartup() {

    iGmo.options = MQC.MQGMO_WAIT | MQC.MQGMO_CONVERT | 
                  MQC.MQGMO_NO_SYNCPOINT | MQC.MQGMO_FAIL_IF_QUIESCING;
    iPmo.options = MQC.MQPMO_NO_SYNCPOINT | MQC.MQPMO_FAIL_IF_QUIESCING
              | MQC.MQPMO_SET_IDENTITY_CONTEXT;

    MQEnvironment.hostname = iConstants.getHostName();
    MQEnvironment.channel  = iConstants.getChannelName();

//  Connect to the specified queue manager
    try {
      iQmgr = new MQQueueManager( iConstants.getQueueManagerName() );

      int openOptions = MQC.MQOO_OUTPUT | MQC.MQOO_FAIL_IF_QUIESCING
                | MQC.MQOO_SET_IDENTITY_CONTEXT;
// Open request queue
      iRequestQueue = iQmgr.accessQueue( 
                                    iConstants.getRequestQueueName(),
                                     openOptions, null, null, null);

      openOptions = MQC.MQOO_INPUT_SHARED | MQC.MQOO_FAIL_IF_QUIESCING;

// Open reply queue
      iReplyQueue = iQmgr.accessQueue( iConstants.getReplyQueueName(),
                                      openOptions, null, null, null);
    } catch (Exception ex) {
      ex.printStackTrace();
    }
  }

/**
 * This method places a message on a queue and waits for a reply 
 * on the reply queue. It returns the time taken or -1 if it fails.
 */
  public long sendAndReceive(String ioutMessage, long timeOut) {

    iGmo.waitInterval = (int) timeOut;

    MQMessage outMess = applyDefaults(ioutMessage);
    long timeStart = System.currentTimeMillis();
    try {
      iRequestQueue.put(outMess, iPmo);
      iReplyQueue.get( outMess, iGmo, 
                              iConstants.getMaximumMessageLength() );

      return System.currentTimeMillis() - timeStart;

    } catch (Exception e) {
      System.out.println("Put/Get error : " + e);
    }
    return -1;
  }

/**
*   Close down the mq queues and connection
*/
  public void finalize() {

    try {
      iReplyQueue.close();
      iRequestQueue.close();
      iQmgr.disconnect();
    } catch (MQException tooLateToWorry) {}
  }
}


package com.dmitri.mqstress;
import java.awt.*;
import com.sun.java.swing.*;

/**
*   @author Dmitri
*
*  This class is the actual control object for the MQStress test 
*  package. It calls the MQService class and reports results back 
*  to the main class by supplying a component to hold results.
*
*   In this implementation, a simple JLabel is supplied to show 
*   test results as they happen. This gives an on-line view that 
*   immediately highlights problem areas. This implementation does 
*   not store results of any kind. It could be appropriate to extend 
*   this class to do just that, enabling extensive analysis of 
*   results at a later stage.
*/
public class MQStressThread implements Runnable {

  private JLabel iRenderer = new JLabel("Inactive");
  private MQService iService = new MQService();

  private boolean iStopRequested = false;  // Flag to determine 
                                      // whether the control module
                                      // has requested a stop.

  private MQConstants iConstants = MQConstants.getInstance();

  private long [] iAcceptable;
  private long [] iFailure;
  private long [] iPace;
  private String [] iMessage;

  private Color iClrAcceptable = new Color(0, 70, 0); // Dark green
  private Color iClrWarning = new Color(170, 170, 0);  // Yellowish
  private Color iClrFail = Color.red;
  private Color iClrDefault = Color.black;

/**
*    Constructor. Initializes local copies of runtime parameters to 
*    avoid delays when running.
*/
  public MQStressThread() {

    iRenderer.setPreferredSize(new Dimension(60,20));
    iRenderer.setForeground(iClrDefault);
    int messNo = iConstants.getNumberOfMesages();
    iAcceptable = new long[messNo];
    iFailure = new long[messNo];
    iPace = new long[messNo];
    iMessage = new String[messNo];

    for (int i = 0; i < messNo; i++) {
      iAcceptable[i] = iConstants.getAcceptableTime(i);
      iFailure[i] = iConstants.getFailureTime(i);
      iPace[i] = iConstants.getPaceTime(i);
      iMessage[i] = iConstants.getMessage(i);
    }
    iService.mqStartup();
  }

/**
*    This method returns the 'renderer' that is used to display the 
*    results of the stress test as they happen.
*/
  public Component getRenderer() {
    return iRenderer;
  }

/**
*    Starts the stress test. Records response times and paces 
*    according to pre-set parameters.
*/
  public void run() {

    iStopRequested = false;
    int numberOfRepetitions = iConstants.getNumberOfRepetitions();
    int numberOfMessages = iConstants.getNumberOfMesages();

    long startTime = 0;
    long response = 0;
    long sleepTime = 0;

    for (int i = 0; i < numberOfRepetitions && !iStopRequested; i++) {

      for (int j = 0; j < numberOfMessages && !iStopRequested; j++) {

        startTime = System.currentTimeMillis();
        response = iService.sendAndReceive(iMessage[j], iPace[j]);
                                  // Maximum time allowed is pace time.

        // Could log to disk here in future extensions.

        if (response < 0) {       // Complete failure
          iRenderer.setForeground(iClrFail);
          iRenderer.setText("Failure");

        } else if (response < iAcceptable[j]) {
          iRenderer.setForeground(iClrAcceptable);
          iRenderer.setText(Long.toString(response));

        } else if (response < iFailure[j]) {
          iRenderer.setForeground(iClrWarning);
          iRenderer.setText(Long.toString(response));

        } else if (response > iFailure[j]) {
          iRenderer.setForeground(iClrFail);
          iRenderer.setText(Long.toString(response));

        }
        sleepTime = iPace[j] - ( System.currentTimeMillis() 
                                                   - startTime );
        try {
           Thread.sleep(sleepTime);
        } catch (Exception ignored) {}  // Could be an interrupt or 
                                        // negative sleep
      }
    }
  }
/**
*    Stops the stress test
*/
  public void stopTest() {
    iStopRequested = true;
    iRenderer.setText("Stop");
    iRenderer.setForeground(iClrDefault);
  }
/**
*    Prepares for exiting
*/
  public void exit() {
    stopTest();
  }
}


